/* r821pre
 * with "+ Switches" add two switchfunctions:
 * every activation of the trainer switch toggles CH8 value between +100 and -100 (FlipFlop)
 * Momentary Activation of AIL switch sets CH7 to + 100, momentary activation of ELE switch resets CH7 to -100 (R-S FFlop)
 * set limits in Limit Menu to defaults
 * change two position switch to be used in the source field (sGEA) without additional switch entry
 * applyTemplate lscht jetzt auch die kompletten TrainerSetups, die Heli-Setups, die logical Switches, die CustomSwitches
 * in the trainer template, add "Enable PPSIM=ON" for use as slave 
 * add Template for "Trainer(Master) Switch Setup"
 *   setup the Trainer Menu to be activated by switch AIL
 *   have control over the student if any of the trainer sticks is moved out of
 *   its default/neutral position
 *   give control to the student as long as all four sticks are in their default position
 *   TRN switch gives unconditional control of throttle channel to the student
 *
 * change templates to T-Tail, V-Tail, Delta, Aileron, ...
 * change sticky throttle cut to use voice switch function "X"
 * modified sequence of templates menu, add Switches template
 * start template name with "+" or "R" (add or replace)
 * Replace templates (R ...) clear custom switches.
 * set default number of voice switches to 16
 * Throttle and aileron added to V-Tail
 * Throttle and rudder added to Delta
 * Delta with 30% differential for ailerons
 * mix weight for mixed surfaces reduced to 50%
 * changed direction in delta and V-Tail mix
 * new Switches template adds 2Pos and 3Pos switch. 3Pos switch with slow up/down.
 * 3Pos switch mix to use MIX_3POS. Saves one mixer line. 
 * changed Servo test template to use timer. Saves 126Byte Flash ram
 * reverse throttle cut switch active position
 *
 * Author - Erez Raviv <erezraviv@gmail.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * ============================================================
 * Templates file
 *
 * eccpm
 * crow
 * throttle cut
 * flaperon
 * elevon
 * v-tail
 * throttle hold
 * Aileron Differential
 * Spoilers
 * Snap Roll
 * ELE->Flap
 * Flap->ELE
 *
 *
 *
 * =============================================================
 * Assumptions:
 * All primary channels are per modi12x3
 * Each template added to the end of each channel
 *
 *
 *
 */

#include "er9x.h"
#include "templates.h"
#include "language.h"

/*
   Der Schler erhlt Kontrolle wenn der Trainer den Schalter AIL aktiviert und alle seine Knppel in default Position
   sind, d.h. RUD, ELE, AIL neutral und THR auf low.
   Der Lehrer bernimmt die Kontrolle fr eine beliebige Achse, sobald er den Knppel von deren default Position wegbewegt.
   Er gibt die Kontrolle zurck an den Schler, indem er seinen Knppel zurck in default Position bringt. 
   Dabei wird jedoch bei einem motorisiertem Modell zwangslufig der Motor kurzfristig heruntergefahren (THR < -95) ehe die
   THR Position des Schlers Kontrolle erhlt.   
   Mit Hilfe des TRN Schalters kann der Trainer jedoch kurzfristig bedingungslos die Kontrolle fr den Throttle Kanal an
   den Schler bergeben, dabei seinen Throttle Steuerknppel in default Stellung bringen und danach den TRN Schalter
   loslassen. Damit bleibt die Kontrolle beim Schler und es kommt zu keinen ungewollten Throttle Vernderung bei der 
   bergabe an den Schler.
*/

#ifndef NO_TEMPLATES

	const static prog_char APM string_1[] = STR_T_T_TAIL   ;
	const static prog_char APM string_2[] = STR_T_V_TAIL    ;			/* ReSt */
	const static prog_char APM string_3[] = STR_T_DELTA     ;
	const static prog_char APM string_4[] = STR_T_AILERON   ;			/* ReSt */
	const static prog_char APM string_5[] = STR_T_SWITCHES  ;
	const static prog_char APM string_6[] = STR_T_TCUT      ;
	const static prog_char APM string_7[] = STR_T_STICK_TCUT;

	const static prog_char APM string_8[] = STR_T_TRAINER;		// added in en.h	/* ReSt */

	const static prog_char APM string_9[] = STR_T_SERVO_TEST;

#ifndef NO_HELI			//  = if Heli
	const static prog_char APM string_10[] = STR_T_HELI_SETUP;
	const static prog_char APM string_11[] = STR_T_GYRO      ;
	const prog_char *const n_Templates[12] PROGMEM = {
	string_1,
      string_2,
      string_3,
      string_4,
      string_5,
      string_6,
      string_7,
      string_8,
      string_9,
	string_10,
	string_11

#else
        const prog_char *const n_Templates[9] PROGMEM = {
      string_1,
      string_2,
      string_3,
      string_4,
      string_5,
      string_6,
      string_7,
	string_8,
	string_9
#endif		//NO_HELI
};

static MixData* setDest(uint8_t dch)
{
    uint8_t i = 0;
    MixData *md = &g_model.mixData[0];

    while ((md->destCh<=dch) && (md->destCh) && (i<MAX_MIXERS)) i++, md++;
    if(i==MAX_MIXERS) return &g_model.mixData[0];

    memmove(md+1, md, (MAX_MIXERS-(i+1))*sizeof(MixData) );
    memset( md, 0, sizeof(MixData) ) ;
    md->destCh = dch;
    md->weight = 100 ;
#ifndef V2
		md->lateOffset = 1 ;
#endif
    return md ;
}

//#ifdef NO_TEMPLATES
//inline
//#endif 

void setSwitch(uint8_t idx, uint8_t func, int8_t v1, int8_t v2)
{
#ifdef V2
	CxSwData *cs = &g_model.customSw[idx-1] ;
#else
	CSwData *cs = &g_model.customSw[idx-1] ;
#endif // V2
  cs->func = func ;
  cs->andsw = 0 ;
  cs->v1   = v1 ;
  cs->v2   = v2 ;
}


void clearMixes()
{
    memset(g_model.mixData,0,sizeof(g_model.mixData)); //clear all mixes
// clear custom switches		
   for(uint8_t i=1; i<13; i++) 
	{
        setSwitch(i,CS_OFF, CS_OFF, 0);
	  g_model.customSw[i-1].andsw   = 0;   //remove a possible switch in param col 4		/* ReSt */
	}
#ifndef V2
	g_model.numVoice = 16 ;									/* ReSt */
	for(uint8_t i=0; i<16; i++) 								// clear the 16 Safety Switches
	{
		SafetySwData *sd = &g_model.safetySw[i];
		sd->opt.vs.vswtch = 0 ;
		sd->opt.vs.vval = 0 ;
		sd->opt.vs.vmode = 0 ;
	}	

	for(uint8_t i=0; i<8; i++) 								// clear extra Voice Switches 17-24
	{		
		SafetySwData *sd = &g_model.safetySw[i];
		sd = &g_model.xvoiceSw[i];
		sd->opt.ss.mode = 0 ;
		sd->opt.ss.swtch = 0 ;
		sd->opt.ss.val = 0 ;	
	}
#endif

	clearCurves();
	clearLimits();
	clearTrainerSetup();
	clearHeli();
}


void clearLimits()
{
	for(uint8_t i=0; i<NUM_CHNOUT; i++)
	{
      	g_model.limitData[i].offset = 0 ;	

		LimitData *ld = &g_model.limitData[i] ;
		FORCE_INDIRECT(ld) ;
		ld->min = 0;
	      ld->max = 0;
		ld->reverse = 0 ;
	}
}

void clearHeli()
{
	g_model.swashType = 0 ;
	g_model.swashCollectiveSource = 0 ;
	g_model.swashRingValue = 0 ;
	g_model.swashInvertELE = 0 ;
	g_model.swashInvertAIL = 0 ;
	g_model.swashInvertCOL = 0 ;
}

#ifndef NO_TEMPLATES
void clearCurves()
{
    memset(g_model.curves5,0,sizeof(g_model.curves5)); //clear all curves
    memset(g_model.curves9,0,sizeof(g_model.curves9)); //clear all curves
}


void clearTrainerSetup()
{
	g_model.traineron=0;								// Trainer mode off

	// clear Trainer Setup
	for (uint8_t i=0; i<4; i++) 					
	{
#ifdef V2
		volatile V2TrainerMix *td = &g_eeGeneral.trainer.mix[i];
#else
		volatile TrainerMix *td = &g_eeGeneral.trainer.mix[i];
#endif
		for (uint8_t j=0; j<4; j++)
		{
      		switch(j)
			{
				case 0 :
					 td->mode = 0 ; 			// off
					break ;

				case 1 :
					break ;
	
				case 2 :
					td->srcChn = 0 ; 			// i = CH1 ... CH4
					break ;
				
				case 3 :
#ifdef V2
					td->swtch = 0; 			// ---
#else
					td->swtch = DSW_THR-1; 		// ---
#endif
					break ;
			}
		}
	}

}
#endif		//NO_TEMPLATES

//-------------------------------------------------------

static void setCurve(uint8_t c, const prog_int8_t *ar )
{
	int8_t *p ;
	
	p = g_model.curves5[c] ;
  for(uint8_t i=0; i<5; i++)
	{
		*p++ = pgm_read_byte(ar++) ;
	}
}

#ifndef FIX_MODE
NOINLINE uint8_t convert_mode_helper(uint8_t x)
{
    return pgm_read_byte(modi12x3 + g_eeGeneral.stickMode*4 + (x) - 1) ;
}
#endif

#ifndef NO_TEMPLATES
#ifndef NO_HELI
const prog_int8_t heli_ar1[] PROGMEM = {-100, 20, 30, 70, 90};
const prog_int8_t heli_ar2[] PROGMEM = {80, 70, 60, 70, 100};
const prog_int8_t heli_ar3[] PROGMEM = {100, 90, 80, 90, 100};
const prog_int8_t heli_ar4[] PROGMEM = {-30, -15, 0, 50, 100};
const prog_int8_t heli_ar5[] PROGMEM = {-100, -50, 0, 50, 100};
#endif
#endif

/*
#ifdef NO_TEMPLATES
	void applyTemplate()
#else
	void applyTemplate(uint8_t idx)
#endif		*/	//NO_TEMPLATES



#ifdef NO_TEMPLATES
void applyTemplate()
#else
void applyTemplate(uint8_t idx)
#endif
{
#ifndef NO_TEMPLATES
    MixData *md = &g_model.mixData[0];
#endif

#define ICC(x) icc[(x)-1]

    uint8_t icc[4] ;
		uint8_t bch = pgm_read_byte(bchout_ar + g_eeGeneral.templateSetup) ;
    for ( uint8_t i = 4 ; i > 0 ; i -= 1 )
		{
			icc[bch & 3] = i ;
			bch >>= 2 ;
		}

    uint8_t j = 0;

// All templates clear the existing model mixes, switches and curves except these four
    switch (idx) {
    	case 4:		//Switches	
    	case 5:		//T-Cut
    	case 6:		//Sticky T-Cut
    	case 7:		//Intelligent Trainer Switch
	case 10:		//Gyro setup

		break;
	default:
        clearMixes();	//Clear Mixes, Switches, Curves and Trainer Setup
	  break;
	}

//*************************************************************
    //T_Tail    Simple 4-Ch
    if(idx==j++) 
    {
        md=setDest(ICC(STK_AIL));  md->srcRaw=CM(STK_RUD);					// Chan1
        md=setDest(ICC(STK_ELE));  md->srcRaw=CM(STK_ELE);	md->weight= -100;		// Chan2
        md=setDest(ICC(STK_THR));  md->srcRaw=CM(STK_THR);					// Chan3
        md=setDest(ICC(STK_RUD));  md->srcRaw=CM(STK_AIL);					// Chan4
    }
//*************************************************************
    //V-Tail
    if(idx==j++) 
    {
		// CH 3					Stick
		md=setDest(ICC(STK_THR));	 md->srcRaw=CM(STK_THR);
		// CH 4					Stick
		md=setDest(ICC(STK_AIL));	 md->srcRaw=CM(STK_RUD);
		
		// CH2					Stick
		md=setDest(ICC(STK_ELE));  md->srcRaw=CM(STK_ELE);  md->weight= -50;
		md=setDest(ICC(STK_ELE));  md->srcRaw=CM(STK_AIL);  md->weight= 50;

		// CH1					Stick
		md=setDest(ICC(STK_RUD));  md->srcRaw=CM(STK_ELE);  md->weight= -50;
		md=setDest(ICC(STK_RUD));  md->srcRaw=CM(STK_AIL);  md->weight= -50;
    }
//*************************************************************
    //Delta
    if(idx==j++)
    {
		// CH 3					Stick
	  md=setDest(ICC(STK_THR));	 md->srcRaw=CM(STK_THR);	

		// CH 1					Stick
        md=setDest(ICC(STK_RUD)); md->srcRaw=CM(STK_ELE);  md->weight= -50; 
        md=setDest(ICC(STK_RUD)); md->srcRaw=CM(STK_AIL);  md->weight=50; md->differential= 1; md->curve= 30;/* ReSt */

		// CH2					Stick
	  md=setDest(ICC(STK_ELE)); md->srcRaw=CM(STK_ELE);  md->weight= -50;
        md=setDest(ICC(STK_ELE)); md->srcRaw=CM(STK_AIL); md->weight= -50; md->differential= 1; md->curve= -30;/* ReSt */

		// CH4					Stick
        md=setDest(ICC(STK_AIL)); md->srcRaw=CM(STK_RUD);
    }

//*************************************************************
    // Aileron
    if(idx==j++)
    {
		// CH1					Stick
        md=setDest(ICC(STK_RUD)); md->srcRaw=CM(STK_AIL);  md->weight=100; md->differential= 1; md->curve= 30;/* ReSt */

		// CH 2					Stick
		md=setDest(ICC(STK_ELE));	 md->srcRaw=CM(STK_ELE); md->weight= -100; 

		// CH 3					Stick
		md=setDest(ICC(STK_THR));	 md->srcRaw=CM(STK_THR);

		// CH4					Stick
        md=setDest(ICC(STK_AIL)); md->srcRaw=CM(STK_AIL); md->weight= -100; md->differential= 1; md->curve= -30;/* ReSt */
    }

//*************************************************************
    // Switches
    if(idx==j++)
    {
		md=setDest(5);  md->srcRaw=MIX_3POS; md->speedUp=3; md->speedDown=3;	// Chan5
//		md=setDest(6);  md->srcRaw=MIX_3POS; md->switchSource=5;	
//This should work, but gets error: MixData has no member named switchSource
// instead of, sw23pos does work !!! 0=sIDx, 1=sTHR, 2=sRUD, 3=sELE, 4=sAIL, 5=sGEA, 6=sTRN
 
		md=setDest(6);  md->srcRaw=MIX_3POS; md->sw23pos=5;				// Chan6 sGEA

//************************************

// Logic Switch for SET / Reset latch. AIL = Set,  ELE = Reset
      	setSwitch(6,CS_LATCH, DSW_AIL,DSW_ELE );					//L6: RS_FF   AIL ELE

// Logic switch for toggeling Flip-Flop. TRN toggles status
      	setSwitch(7,CS_FLIP, DSW_TRN,-DSW_SW7 );					//L7: D-LATCH TRN !L7  

// Channel Mix for SET / RESET Latch
		md=setDest(7);  md->srcRaw=MIX_FULL; md->swtch=15;	

// Channel mix for Flip-Flop
		md=setDest(8);  md->srcRaw=MIX_FULL; md->swtch=16;	
//************************************
    }


    //T-Cut
    if(idx==j++)
    {
#ifdef V2
			V2SafetySwData *sd = &g_model.safetySw[ICC(STK_THR)-1] ;
			sd->mode = 0 ;
			sd->swtch = DSW_THR ;
			sd->val = g_model.throttleIdle ? 0 : -100 ;
#else
	g_model.numVoice = 13 ;									/* ReSt */
    	SafetySwData *sd = &g_model.safetySw[ICC(STK_THR)-1] ;
			sd->opt.ss.mode = 0 ;
			sd->opt.ss.swtch = DSW_THR ;
			sd->opt.ss.val = g_model.throttleIdle ? 0 : -100 ;
#endif // V2
    }
//*************************************************************
    //sticky t-cut
    if(idx==j++)
    {
#ifdef V2
			V2SafetySwData *sd = &g_model.safetySw[ICC(STK_THR)-1] ;
			sd->mode = 1 ;
			sd->swtch = -DSW_THR ;
			sd->val = g_model.throttleIdle ? 0 : -100 ;
#else
	g_model.numVoice = 13 ;									/* ReSt */
    	SafetySwData *sd = &g_model.safetySw[ICC(STK_THR)-1] ;
			sd->opt.ss.mode = 3 ;
			sd->opt.ss.swtch = -DSW_THR ;
			sd->opt.ss.val = g_model.throttleIdle ? 0 : -100 ;
#endif // V2
    }

//*************************************************************
    //intelligent Trainer Switch
	// each of the Switches checks the position of one stick axis 
	// every switch enables the local stick to override the slave, when its value is out of bounds 

    if(idx==j++)
    {
	g_model.traineron=1;								// activate Trainer mode
	g_eeGeneral.enablePpmsim=1;							//activate PPSIM in case of slave

	// sets Custom Switches 1 - 5
      setSwitch(CM(STK_RUD),CS_ANEG, CM(STK_RUD), 5);				//Sw1: |v|<ofs RUD 5
      setSwitch(CM(STK_ELE),CS_ANEG, CM(STK_ELE), 5);				//Sw2: |v|<ofs ELE 5
      setSwitch(3,CS_OR, DSW_SW5, DSW_TRN);					//Sw3: OR SW5 TRN 
      setSwitch(CM(STK_AIL),CS_ANEG, CM(STK_AIL), 5);				//Sw4: |v|<ofs AIL 5
      setSwitch(5,CS_VNEG, CM(STK_THR), -95);					//Sw5:  <  THR -95

	// AND the AIL switch to all five custom switches
	for (uint8_t i=0; i<5; i++) 					// must be here in a separate loop
	{
		g_model.customSw[i].andsw   = DSW_AIL;   				//Sw1-5  AND AIL
	}

	// setup Trainer screen
	for (uint8_t i=0; i<4; i++) 					
	{
#ifdef V2
		volatile V2TrainerMix *td = &g_eeGeneral.trainer.mix[i];
#else
		volatile TrainerMix *td = &g_eeGeneral.trainer.mix[i];
#endif
		for (uint8_t j=0; j<4; j++)
		{
      		switch(j)
			{
				case 0 :
					 td->mode = 2 ; 			// replace  :=
					break ;

				case 1 :
#ifdef V2
					td->studWeight = 100; 		// = +100%
#else
					td->studWeight = 31; 		// = +100%
#endif
					break ;
	
				case 2 :
					td->srcChn = i ; 			// CH1 ... CH4
					break ;
				
				case 3 :
					td->swtch = DSW_SW1+i; 		// L1 ... L4
					break ;
			}
		}
	}

    }


//*************************************************************


    //Servo Test
    if(idx==j++)
    {
        md=setDest(1); md->srcRaw=MIX_FULL; md->swtch=DSW_SW1; md->mltpx=MLTPX_ADD; md->speedUp=3; md->speedDown=3;
	  setSwitch(0x1,CS_TIME, 3, 3); 
    }

//--------------------------------------------------------------------
#ifndef NO_HELI
    //Heli Setup
    if(idx==j++)
    {

        //Set up Mixes
        //3 cyclic channels
        md=setDest(1);  md->srcRaw=MIX_CYC1;
        md=setDest(2);  md->srcRaw=MIX_CYC2;
        md=setDest(3);  md->srcRaw=MIX_CYC3;

        //rudder
        md=setDest(4);  md->srcRaw=CM(STK_RUD);

        //Throttle
        md=setDest(5);  md->srcRaw=CM(STK_THR); md->swtch= DSW_ID0; md->curve=CV(1); md->carryTrim=TRIM_OFF;
        md=setDest(5);  md->srcRaw=CM(STK_THR); md->swtch= DSW_ID1; md->curve=CV(2); md->carryTrim=TRIM_OFF;
        md=setDest(5);  md->srcRaw=CM(STK_THR); md->swtch= DSW_ID2; md->curve=CV(3); md->carryTrim=TRIM_OFF;
        md=setDest(5);  md->srcRaw=MIX_MAX;      md->weight=-100; md->swtch= DSW_THR; md->mltpx=MLTPX_REP;

        //gyro gain
        md=setDest(6);  md->srcRaw=MIX_FULL; md->weight=30; md->swtch=-DSW_GEA;

        //collective
        md=setDest(11); md->srcRaw=CM(STK_THR);  md->weight=70; md->swtch= DSW_ID0; md->curve=CV(4); md->carryTrim=TRIM_OFF;
        md=setDest(11); md->srcRaw=CM(STK_THR);  md->weight=70; md->swtch= DSW_ID1; md->curve=CV(5); md->carryTrim=TRIM_OFF;
        md=setDest(11); md->srcRaw=CM(STK_THR);  md->weight=70; md->swtch= DSW_ID2; md->curve=CV(6); md->carryTrim=TRIM_OFF;

        g_model.swashType = SWASH_TYPE_120;
        g_model.swashCollectiveSource = CH(11);

        //Set up Curves
        setCurve(CURVE5(1),heli_ar1);
        setCurve(CURVE5(2),heli_ar2);
        setCurve(CURVE5(3),heli_ar3);
        setCurve(CURVE5(4),heli_ar4);
        setCurve(CURVE5(5),heli_ar5);
        setCurve(CURVE5(6),heli_ar5);
    }

    //Gyro Gain
    if(idx==j++)
    {
        md=setDest(6);  md->srcRaw=STK_P2; md->weight= 50; md->swtch=-DSW_GEA; md->sOffset=50;
        md=setDest(6);  md->srcRaw=STK_P2; md->weight=-50; md->swtch= DSW_GEA; md->sOffset=-50;
    }

#endif		// NO_HELI


    STORE_MODELVARS;
    eeWaitComplete() ;

#else					// Templates=No
void applyTemplate()
{

#endif			//NO_TEMPLATES

}


